#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <qlib2.h>

#include "segy_reel.h"
#include "externals.h"

void swab_bin_reel_header (SEGYBREELHEAD *bin_reel_header);

/************************************************************************/
/*  read_segy_reel:							*/
/*	Read SEGY reel headers.  Fill in binary reel header.		*/
/*  return:								*/
/*	Number of traces in file (1 if no reel header).			*/
/************************************************************************/
int read_segy_reel 
   (FILE	*input,
    SEGYBREELHEAD **pbin_reel_header,
    int		segy_fmt)
{
    SEGYDREELHEAD char_reel_header;
    SEGYBREELHEAD *bin_reel_header;
    int ntraces;

    *pbin_reel_header = NULL;
    bin_reel_header = (SEGYBREELHEAD *)malloc(sizeof(SEGYBREELHEAD));
    switch (segy_fmt) {
      case BERKELEY_FORMAT:
	/* Skip the text descriptive reel header record.			*/
	if (fread(&char_reel_header,1,sizeof(SEGYDREELHEAD),input)!=sizeof(SEGYDREELHEAD)) {
	    fprintf(stderr, "Error reading in a SEG-Y Descriptive reel header.\n");
	    exit(1);
	}
	/* Read the binary reel header.					*/
	if (fread(bin_reel_header,1,sizeof(SEGYBREELHEAD),input)!=sizeof(SEGYBREELHEAD)) {
	    fprintf(stderr, "Error reading in a SEG-Y Binary reel header.\n");
	    exit(1);
	}
	if (my_endian != SEED_BIG_ENDIAN) {
	    swab_bin_reel_header(bin_reel_header);
	}
	ntraces = bin_reel_header->kntr;
	break;
      case PASSCAL_FORMAT:
	memset((void *)bin_reel_header, 0, sizeof(SEGYBREELHEAD));
	ntraces = 1;
	break;
      default:
	fprintf (stderr, "Invalid segy format: %d\n", segy_fmt);
	exit(1);
	break;
    }
    *pbin_reel_header = bin_reel_header;
    return (ntraces);
}

void swab_bin_reel_header (SEGYBREELHEAD *bin_reel_header)
{
    swab4(&(bin_reel_header->kjob));
    swab4(&(bin_reel_header->kline));
    swab4(&(bin_reel_header->kreel));
    swab2(&(bin_reel_header->kntr));
    swab2(&(bin_reel_header->kaux));
    swab2(&(bin_reel_header->sr));
    swab2(&(bin_reel_header->kfldsr));
    swab2(&(bin_reel_header->knsamp));
    swab2(&(bin_reel_header->kfsamp));
    swab2(&(bin_reel_header->kform));
    swab2(&(bin_reel_header->kmfold));
    swab2(&(bin_reel_header->ksort));
    swab2(&(bin_reel_header->kvsum));
    swab2(&(bin_reel_header->kswfrs));
    swab2(&(bin_reel_header->kswfre));
    swab2(&(bin_reel_header->kmssw));
    swab2(&(bin_reel_header->kswtyp));
    swab2(&(bin_reel_header->kswtr));
    swab2(&(bin_reel_header->kmsbst));
    swab2(&(bin_reel_header->kmsest));
    swab2(&(bin_reel_header->ksttyp));
    swab2(&(bin_reel_header->kcorcd));
    swab2(&(bin_reel_header->kbgrcd));
    swab2(&(bin_reel_header->kartyp));
    swab2(&(bin_reel_header->kunits));
    swab2(&(bin_reel_header->kpol));
    swab2(&(bin_reel_header->kvpol));
    /* Additional unused elements used by LBL. */
    swab2(&(bin_reel_header->nyear));
    swab2(&(bin_reel_header->nday));
    swab2(&(bin_reel_header->nhour));
    swab2(&(bin_reel_header->nmin));
    swab2(&(bin_reel_header->nsec));
    swab2(&(bin_reel_header->nmsec));
}
